
function dbiScriptApplication() {

  app.dbiScriptProperties.rebuildDatabase = true;  // Must also set "allowRebuildDatabase = True" in clinic.cfg

  app.dbiScriptProperties.dataConfig = "../data/clinic.cfg"
  app.dbiScriptProperties.dataTables.fetchAll = false;

  app.addDefinitionTable("dfnmst",["descr","varchar(255)"],["srcdbf","varchar(15)"],["srcfld","varchar(15)"],{additionalFields:[["code","varchar(5)"]]});
  app.addDefinitionTable("icd9mst",["descr","varchar(255)"],{additionalFields:[["code","varchar(5)"]]});
  app.addDefinitionTable("prcmst",["descr","varchar(255)"],{additionalFields:[["code","varchar(5)"]]});
  app.addFileTable("filemst","srcid",["filename","varchar(255)"],["userid","varchar(20)"],["action","varchar(10)"],["occurred","datetime"],["srcdbf","varchar(15)"],["srcfld","varchar(15)"]);
  app.addLinkTable("linkmst","parentid","childid",["order","tinyint"],["descr","varchar(50)"]);
  app.addPhotoTable("photomst","srcid",["filename","varchar(255)"],["descr","varchar(255)"],["order","tinyint"],["srcdbf","varchar(15)"],["srcfld","varchar(15)"]);

  var o = app.gui.appendDataTable("patient","Patients","Patient");
  o.dbiScriptProperties.dataOrder = "lname,fname,mi";
  o.dbiScriptProperties.fields.email.subject = "Test";
  o.dbiScriptProperties.fields.email.body = "This is a test.";
  o.appendRowState("inactv")
  o.appendIconlist("icons","Icons","images",["a.png","b.png","c.png","d.png"]);
  o.appendSelect(["sal","varchar(4)",{indexed:true}],"Sal",{Mr:"Mr",Mrs:"Mrs",Ms:"Ms",Miss:"Miss",Dr:"Dr",Rev:"Rev"});
  o.appendData(["fname","varchar(15)",{indexed:true}],"First Name");
  o.appendData(["mi","varchar(1)",{indexed:true}],"MI",{forceCase:"UPPER"});
  o.appendData(["lname","varchar(25)",{indexed:true}],"Last Name");
  o.appendSelect(["suffix","varchar(3)",{indexed:true}],"Suffix",{"Jr":"Jr","Sr":"Sr",II:"II",III:"III",IV:"IV"});
  o.appendData(["address1","varchar(35)",{indexed:true}],"Address");
  o.appendData(["address2","varchar(35)",{indexed:true}],"Address ...");
  o.appendQuickpick(["city","varchar(25)",{indexed:true}],"City");
  o.appendQuickpick(["state","varchar(2)",{indexed:true}],"State");
  o.appendData(["zip","varchar(10)",{indexed:true}],"Zip");
  o.appendData(["stock","varchar(4)",{indexed:true}],"Stock Symbol");
  o.appendWebServiceRequest("stockPrice","Stock Price","http://quote.yahoo.com/d/quotes.csv",{allowManualRefresh:true,httpMethod:"GET",serverErrorDisplay:"console",textAlign:"right"}); // note: Yahoo returns the web service data in csv format but since we're only receiving a single value, we can use the "string" (default) WebServiceRequest type
  o.appendPhotoAlbum("photo1","Patient Album 1","../data/images/clinic/patient",["photomst","PATIENT","PHOTO1"]);
  o.appendPhotoAlbum("photo2","Patient Album 2","../data/images/clinic/patient1",["photomst","PATIENT","PHOTO2"]);
  o.drilldown.appendData(["ssn","varchar(9)"],"SSN",{inputMask:"[999-999-999]"});
  o.drilldown.appendRadio(["gender","varchar(1)"],"Gender",{M:"Male",F:"Female"},{isRequired:true});
  o.drilldown.appendData(["dob","datetime"],"Date of Birth");
  o.drilldown.appendAge("dob","Age");
  o.drilldown.appendBoolean("billptonly","Bill Patient Only");
  o.drilldown.appendBoolean("sndstmnt","Send Statement");
  o.drilldown.appendData(["hphone","varchar(10)"],["Home","Home Phone Number"],{inputMask:"![(999)] [999-9999]"});
  o.drilldown.appendData(["cellphone","varchar(10)"],["Cell","Cell Number"],{inputMask:"![(999)] [999-9999]"});
  o.drilldown.appendData(["fax","varchar(10)"],["Fax","Fax Number"],{inputMask:"![(999)] [999-9999]"});
  o.drilldown.groupFields("phonenumbers","Phone Numbers","hphone","cellphone","fax");
  o.drilldown.appendBoolean("asig",["Assignment","Assignment Signature"]);
  o.drilldown.appendBoolean("rs",["Release","Release Signature"]);
  o.drilldown.groupFields("sigs","Signatures","asig","rs");
  o.drilldown.appendReference("sdrmstid",["Primary","Primary Physician"],"sdrmst",["lname","fname","mi"],{type:"poptable"});
  o.drilldown.appendReference("rdrmstid",["Referring","Referring Physician"],"rdrmst",["lname","fname","mi"]);
  o.drilldown.groupFields("drs","Physicians","sdrmstid","rdrmstid");
  o.drilldown.appendSelect("statusid","Marital Status",["dfnmst","PATIENT","STATUSID"],{code:"code"});
  o.drilldown.appendSelect("studentid","Student Status",["dfnmst","PATIENT","STUDENT"],{code:"code"});
  o.drilldown.appendSelect("finclassid","Financial Class",["dfnmst","PATIENT","FINCLASSID"],{code:"code"});
  o.drilldown.appendReference("attorneyid","Attorney","attorney",["lname","fname","mi"]);
  o.drilldown.appendSelect("restypeid","Residence",["dfnmst","PATIENT","RESTYPEID"],{allowNew:true,code:"code",dataOrder:"code asc",isRequired:true});
  o.drilldown.appendSelect("mailcodeid","Mail Code",["dfnmst","PATIENT","MAILCODEID"],{allowNew:true,code:"code",dataOrder:"code asc",isRequired:true});
  o.drilldown.appendSelect("npsrcid","Source",["dfnmst","PATIENT","NPSRCID"],{isRequired:true,code:"code",allowNew:true});
  o.drilldown.appendSelect("relatetoid","Injury Related To",["dfnmst","PATIENT","RELATETOID"],{allowNew:true,code:"code",isRequired:true});
  o.drilldown.appendData(["consdttm","datetime"],"First Consultation");
  o.drilldown.appendData(["xraydttm","datetime"],"Initial X-Ray");
  o.drilldown.appendData(["injurydttm","datetime"],"Injury Illness");
  o.drilldown.appendData(["ssympdttm","datetime"],"Similar Symptom");
  o.drilldown.appendData(["lstseendttm","datetime"],"Last Seen");
  o.drilldown.appendData(["lstmensdttm","datetime"],"Last Menstruation");
  o.drilldown.appendData(["retiredttm","datetime"],"Retirement");
  o.drilldown.appendData(["collectdttm","datetime"],"To Collections");
  o.drilldown.appendData(["inactvdttm","datetime"],"Inactive");
  o.drilldown.appendData(["deathdttm","datetime"],"Death");
  o.drilldown.appendSelect("countryid","Country",["dfnmst","COUNTRY","COUNTRY"],{allowNew:true});
  o.drilldown.appendSelect("stateid","State",["dfnmst","STATE","STATE"],["countryid","linkmst","COUNTRY -> STATE"],{allowNew:true});
  o.drilldown.appendReference("refpatientid","Referred By","patient",["lname","fname","mi"]);
  o.drilldown.appendTreeview("itemid","Tree Item",["dfnmst","PATIENT","TV1"],["linkmst","PATIENT TV1"],{allowDeleteLinkedItems:true,allowNew:true,windowHeight:600,windowWidth:400});
  o.drilldown.appendFileVault("file","Patient File",["filemst","PATIENT","FILE1"],"../data/files/clinic/patient");
  o.drilldown.appendPicklist("icd9ids","Diagnosis","icd9mst",["linkmst","PATIENT ICD9ID"],{allowNew:true,dataOrder:"asc"});
  o.drilldown.appendData(["notes","longtext"],"Notes");
  o.drilldown.appendCopyButton("backup","Backup Patient Data","patient_copy")

  o = app.gui.appendDataTable("insmst","Insurance Companies","Insurance Company");
  o.dataOrder = "name";
  o.appendData(["name","varchar(35)"],"Company Name");
  o.appendSelect("typeid","Type",["dfnmst","INSMST","TYPEID"],{allowNew:true,code:"code",isRequired:true});
  o.appendData(["contact","varchar(25)"],"Contact");
  o.appendData(["phone","varchar(10)"],"Phone Number",{inputMask:"![(999)] [999-9999]"});
  o.appendData(["pext","varchar(5)"],"Ext");
  o.appendMemoPreview("notes","Notes");
  o.drilldown.appendData(["address1","varchar(35)"],["","Address"]);
  o.drilldown.appendData(["address2","varchar(35)"],["","Address ..."],{editSize:"fieldWidth"});
  o.drilldown.appendData(["city","varchar(25)"],"City");
  o.drilldown.appendData(["state","varchar(20)"],"State");
  o.drilldown.appendData(["zip","varchar(10)"],"Zip");
  o.drilldown.groupFields("address","Address","address1","address2","city","state","zip");
  o.drilldown.appendData(["fax","varchar(10)"],"Fax Number",{inputMask:"![(999)] [999-9999]"});
  o.drilldown.appendData(["cellphone","varchar(10)"],"Cell Phone",{inputMask:"![(999)] [999-9999]"});
  o.drilldown.appendEmail(["email","varchar(30)"],"Email Address");
  o.drilldown.appendURL(["website","varchar(30)"],"Website");
  o.drilldown.appendData("ocnano","OCNA #");
  o.drilldown.appendRadio(["bmethod","tinyint"],"Billing Method",{1:"Electronic",2:"HCFA",3:"Invoice",4:"UB92"});
  o.drilldown.appendRadio(["paysystem","tinyint"],"Payment System",{1:"Fee For Service",2:"Capitated"});
  o.drilldown.appendData(["payorid","varchar(10)"],"Payor ID");
  o.drilldown.appendData(["claimoffice","varchar(4)"],"Claim Office");
  o.drilldown.appendReference("submitterid","Submitter","submtmst","name");
  o.drilldown.appendSelect("btypeid","Billing Type",["dfnmst","IBTYPEMST","BTYPEMSTID"],{allowNew:true,code:"code",isRequired:true});
  o.drilldown.appendData(["notes","longtext"],"Notes",{rows:5});

  o = app.gui.appendDataTable("sdrmst","MDs","MD");
  o.dbiScriptProperties.dataOrder = "lname,fname,mi";
  o.appendData(["fname","varchar(15)"],"First Name");
  o.appendData(["mi","varchar(1)"],"MI",{forceCase:"UPPER"});
  o.appendData(["lname","varchar(25)"],"Last Name");
  o.drilldown.appendPicklist("specialtyids","Specialties",["dfnmst","DRMST","CPSPECID"],["linkmst","SDRMST CPSPECID"],{allowNew:true,maxItems:5});

  o = app.gui.appendDataTable("rdrmst","Referring MDs","Referring MD");
  o.dbiScriptProperties.dataOrder = "lname,fname,mi";
  o.appendData(["fname","varchar(15)"],"First Name");
  o.appendData(["mi","varchar(1)"],"MI",{forceCase:"UPPER"});
  o.appendData(["lname","varchar(25)"],"Last Name");

  o = app.gui.appendDataTable("attorney","Attorneys","Attorney");
  o.dbiScriptProperties.dataOrder = "lname,fname,mi";
  o.appendData(["fname","varchar(15)"],"First Name");
  o.appendData(["mi","varchar(1)"],"MI",{forceCase:"UPPER"});
  o.appendData(["lname","varchar(20)"],"Last Name");
  o.appendData(["address1","varchar(35)"],"Address");
  o.appendData(["address2","varchar(35)"],"Address ...");
  o.appendData(["city","varchar(25)"],"City");
  o.appendData(["state","varchar(20)"],"State");
  o.appendData(["zip","varchar(10)"],"Zip");

  o = app.gui.appendDataTable("submtmst","Submitters","Submitter");
  o.appendData("name","Company Name");
  o.appendData("phone","Phone Number",{inputMask:"![(999)] [999-9999]"});
  o.appendURL("website","Website");

  o = app.gui.appendDataTable("policymst","Insurance Policies","Insurance Policy","insmst:insmstid");
  o.appendData(["insgroup","varchar(20)"],"Insurance Group");
  o.appendData(["termdttm","datetime"],"Term");
  o.appendReference("patientid","Policy Owner","patient",["lname","fname","mi"]);
  o.appendData(["contract","varchar(20)"],"Contract");
  o.appendSelect("policytypeid","Policy Type",["dfnmst","POLICYMST","POLICYTYPEID"],{allowNew:true,code:"code",isRequired:true});
  o.appendData(["coverage","varchar(25)"],"Service Code");
  o.appendMoney("deduct","Deductible");
  o.appendMoney("deductelse","Used Deductible");
  o.appendAggregateMoney("total","Total","tranprc","SUM(chargeins+chargepat)");
  o.drilldown.appendData(["copaylvl1","decimal(19,4)"],["Initial","Initial Copay"]);
  o.drilldown.appendSelect(["copaylvl1type","char(1)"],["","Initial Copay Type"],{$:"Dollars","%":"Percent"});
  o.drilldown.appendData(["copaylvl2visits","tinyint"],["After [---] visits","Number of Visits Before Secondary Copay"]);
  o.drilldown.appendData(["copaylvl2","decimal(19,4)"],["","Secondary Copay"]);
  o.drilldown.appendSelect(["copaylvl2type","char(1)"],["","Secondary Copay Type"],{$:"Dollars","%":"Percent"});
  o.drilldown.groupFields("group1","Copay","copaylvl1","copaylvl1type","copaylvl2visits","copaylvl2","copaylvl2type");

  o = app.gui.appendDataTable("patpolicymst","Patient Policies","Patient Policy","patient:patientid,policymst:policymstid",0,1);
  o.appendRadio(["typeid","tinyint"],"Type",{1:"Primary",2:"Secondary",3:"Tertiary",4:"Archive"});
  o.appendSelect("relationid","Relation to Policy Owner",["dfnmst","POLICYMST","RELATIONID"],{allowNew:true,code:"code",isRequired:true});

  o = app.gui.appendDataTable("trandtl","Transactions","Transaction","patpolicymst:patpolicymstid");
  o.dbiScriptProperties.dataOrder = "servfrdttm desc";
  o.appendData(["servfrdttm","datetime"],"Date Opened",{isUniqueWithinForeignKeys:true});
  o.appendData(["claimno","varchar(5)"],"Claim #");
  o.appendData(["fileno","varchar(3)"],"File #");
  o.appendData(["contract","varchar(20)"],"Contract #");
  o.appendData(["coverage","varchar(25)"],"Coverage");
  o.appendBoolean("aa","AA");
  o.appendAggregateMoney("total","Total","tranprc","SUM(chargeins+chargepat)");
  o.appendMemoPreview("notes","Notes",{displaySize:30});
  o.drilldown.appendData(["notes","longtext"],"Notes");

  o = app.gui.appendDataTable("trandtl2","Disability","Disability","trandtl:trandtlid",0,1);
  o.appendData(["totaldisabilityfrom","datetime"],"Total Disability From");
  o.appendData(["totaldisabilityto","datetime"],"Total Disability To");
  o.appendData(["partialdisabilityfrom","datetime"],"Partial Disability From");
  o.appendData(["partialdisabilityto","datetime"],"Partial Disability To");
  o.appendData(["offworkfrom","datetime"],"Off Work From");
  o.appendData(["offworkto","datetime"],"Off Work To");

  o = app.gui.appendDataTable("trandtla","Ambulance","Ambulance","trandtl:trandtlid",0,1);
  o.appendData(["miles","decimal(5,1)"],"Miles");
  o.appendData(["patweight","decimal(5,1)"],"Patient Weight");
  o.appendData(["origin","longtext"],"Origin");
  o.appendData(["dest","longtext"],"Destination");
  o.appendData(["purpose","longtext"],"Purpose");
  o.appendData(["purposestretch","longtext"],"Purpose of Stretcher");

  o = app.gui.appendDataTable("patcharge","Copayments","Copayment","patient:patientid");
  o.dbiScriptProperties.dataOrder = "chargedttm desc";
  o.appendData(["chargedttm","datetime"],"Date Charged");
  o.appendSelect("descriptionid","Description",["dfnmst","PATCHARGE","DESCRIPTIONID"],{allowNew:true});
  o.appendData(["amount","money"],"Amount");

  o = app.gui.appendDataTable("patahst","Allergies","Allergy","patient:patientid");
  o.dbiScriptProperties.dataOrder = "detddttm desc";
  o.appendData(["detddttm","datetime"],"Date Diagnosed");
  o.appendQuickpick("allergyid","Description",["dfnmst","ALLERGYMST","PID"],{allowNew:true,isRequired:true});

  o = app.gui.appendDataTable("patdhst","Medications","Medication","patient:patientid");
  o.dbiScriptProperties.dataOrder = "effdttm desc";
  o.appendData(["effdttm","datetime"],"Date Prescribed");
  o.appendQuickpick("drugid","Description",["dfnmst","DRUGMST","PID"],{allowNew:true,isRequired:true});
  o.appendData(["dosage","varchar(15)"],"Dosage");
  o.appendData(["dispense","varchar(15)"],"Dispense");
  o.appendData(["refill","tinyint"],"Refill");
  o.appendBoolean("selectcode","Select Code");
  o.appendData(["sig","longtext"],"Sig");

  o = app.gui.appendDataTable("tranprc","Procedures","Procedure","trandtl:trandtlid",1);
  o.dbiScriptProperties.dataOrder = "servtodttm desc";
  o.appendData(["servtodttm","datetime"],"Date of Service");
  o.appendQuickpick("prcmstid","Procedure","prcmst");
  o.appendMoney(["chargeins","money"],"Insurance");
  o.appendMoney(["chargepat","money"],"Patient");
  o.appendMoney("charge","Total Charge");

  o = app.gui.addSystemTable("sysparam","System");
  o.appendData("taxrate","Tax Rate",{decimals:4});

}

function format_name(lname,fname,mi) { return lname + ", " + fname + (mi ? " " + mi + "." : ""); }

var patient_attorneyid_format = format_name,
    patient_rdrmstid_format0 = format_name,
    patient_refpatientid_format = format_name,
    patient_sdrmstid_format = format_name,
    policymst_patientid_format = format_name;

function patient_button1_onclick(o) { window.open("http://maps.google.com/maps?q="+(o.address1+"+"+o.city+"+"+o.state).replace(/ /g,"+")); }
function patient_icons_calculate(o) { var a=[],p=/a/i; if(p.test(o.lname))a.push(0); if(p.test(o.address1))a.push(1); if(p.test(o.city))a.push(2); if(p.test(o.state))a.push(3); return a; }
function patient_inactvdttm_updateValue(s,t,i,j,k){if(s == "patient" && t == "inactv")return k ? new Date : undefined; return null;}
function patient_lstmensdttm_isDisplayed(o) { return o.gender == "F"; }
function patient_rdrmstid_format1(lname,fname,mi) { return fname + " " + (mi ? mi + ". " : "") + lname; }
function patient_stockPrice_calculateHttpData(o) { return {s:o.stock,f:"l1"}; }
function patient_stockPrice_processResponse(s) { var lastPrice = parseFloat(s); return isNaN(lastPrice) ? "" : "$" + lastPrice.toFixed(2); }

function insmst_email_setSubject(o) { return "Attn: Claim Office #" + o.claimoffice; }

function trandtl_servfrdttm_newValue() { return new Date; }

function trandtl2_isDisplayed(o) { return o.trandtl.aa; }

function patdhst_effdttm_newValue() { return new Date; }

function tranprc_charge_calculate(o) { return o.chargeins + o.chargepat; }
function tranprc_chargeins_validate(s) { return s > 100000 ? "Charge cannot exceed $100,000" : ""; }
function tranprc_chargeins_verify(s) { return s > 10000 ? "Charge exceeds $10,000." : ""; }
